/*
 * Copyright (c) 2010-2012 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0,
 * and you may not use this file except in compliance with the Apache License Version 2.0.
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Apache License Version 2.0 is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.nexus.plugin.gwt.client;

import com.extjs.gxt.ui.client.data.DataProxy;
import com.extjs.gxt.ui.client.data.DataReader;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.rpc.AsyncCallback;

public class RequestBuilderDataProxy<D> implements DataProxy<D> {

    protected RequestBuilder builder;
    
    /**
     * Creates a new HttpProxy.
     * 
     * @param builder the request builder
     */
    public RequestBuilderDataProxy(RequestBuilder builder) {
      this.builder = builder;
    }

    public void load(final DataReader<D> reader, final Object loadConfig, final AsyncCallback<D> callback) {
      try {

        builder.sendRequest(builder.getRequestData(), new RequestCallback() {
          public void onError(Request request, Throwable exception) {
            callback.onFailure(exception);
          }

          @SuppressWarnings("unchecked")
          public void onResponseReceived(Request request, Response response) {
            if ( !Integer.toString( response.getStatusCode() ).startsWith( "2" ) ) {
              callback.onFailure( new RemoteFailureException( response ) );
              return;
            }
            String text = response.getText();
            try {
              D data = null;
              if (reader != null) {
                data = reader.read(loadConfig, text);
              } else {
                data = (D) text;
              }
              callback.onSuccess(data);
            } catch (Exception e) {
              e.printStackTrace();
              callback.onFailure(e);
            }
          }
        });
      } catch (Exception e) {
        callback.onFailure(e);
      }
    }
}
